#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# [SNIPPET_NAME: Script-Fu Composer]
# [SNIPPET_CATEGORIES: PyGTK, GimpFu]
# [SNIPPET_DESCRIPTION: Complete RAD tool for Gimp to build new script with currently exists]
# [SNIPPET_AUTHOR: Jeremy Czajkowski <jeremy.cz@wp.pl>]
# [SNIPPET_LICENSE: Creative commons <CC-BY-SA>]
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

import os
import re

from gimpfu import *

import gobject
import gtk.gdk
import pygtk

import json
import ConfigParser
import unicodedata

pygtk.require('2.0')


# Dictionaries are sometimes found in other languages as associative memories or associative arrays.
SC_I18N = {}
SC_PREF = {}
SC_FILES= {}
SC_VALS = {}


# Files & Directories
SC_ABSDIR = os.path.dirname(os.path.abspath(__file__)).lower()
SC_RESDIR = os.path.join(SC_ABSDIR ,"_scfiles")
SC_LANGFILE = ""
SC_CONFIGFILE = ""


# Variables
SC_GIMPVERSION = ""
SC_TITLE = "Batch Script Composer"
SC_SAVEMODE_WARNING = "Work in safe mode. Choose you GIMP language correctly /not based on system settings, for example/."

SC_EXCLUDE_TYPE = ['run-mode' , 'image' , 'drawable' ,'layer']
SC_BUTTONS = []
SC_PROCS = []


# Constants
SC_ADD_OPENED_IMAGES = 1001
SC_ADD_RESOURCES = 1002
SC_CLEAR_LIST    = 1003



class ResourceChooserButton(gtk.Fixed):
# @see http://stackoverflow.com/questions/6768779/test-filename-with-regular-expression
# @see http://stackoverflow.com/questions/169008/regex-for-parsing-directory-and-filename
    BG_PATH = os.path.join(SC_RESDIR, "images", "resource_chooser.png")
    KIND = ["dirname", "filename", "resname"]
    REGEX = {"filename": "^[\w\s-]+\.[A-Za-z]{3}$", "dirname": "^(.+)/([^/]+)$" , "resname":"\w"}
    ICON = {"filename": gtk.STOCK_FILE, "dirname": gtk.STOCK_DIRECTORY, "resname": gtk.STOCK_INDEX}
    
    ICON_SIZE = 16
    MENU_LIMIT= 20
    PADDING = 5
  
    def __init__(self, res_kind, win_title=None):
        gtk.Fixed.__init__(self)
        
        self.window_title = win_title
        self.orginal_kind = res_kind
        self.resource_kind = res_kind
        
        background = gtk.Image()
        background.set_from_file(self.BG_PATH)
        self.width = background.get_pixbuf().get_width()
        self.height = background.get_pixbuf().get_height()
        
        self.resource = gtk.Entry()
        self.resource.set_max_length(255)
        self.resource.connect("changed", self.changed)
        
        self.action = gtk.Image()
        try:
            self.action.set_from_stock(self.ICON[res_kind], 2)
        except:
            self.action.set_from_stock(self.ICON["resname"], 2)    
            self.resource_kind = "resname"
        
        action_box = gtk.EventBox()
        action_box.add(self.action)
        action_box.add_events(gtk.gdk.BUTTON_RELEASE_MASK)
        action_box.connect("button-release-event", self.release)
        
        self.add(background)
        self.put(self.resource, self.PADDING, self.PADDING)
        self.put(action_box, self.width - self.ICON_SIZE - self.PADDING * 2, int(self.height * 0.5 - self.ICON_SIZE * 0.5))
        self.set_size_request(self.width, self.height)
        self.show_all()
        self.menu = self.create_menu()
    
    def create_menu(self):
      try:  
        if "pattern" == self.orginal_kind:   
          _, res_list = pdb.gimp_patterns_get_list("")
        elif "gradient" == self.orginal_kind:
          _, res_list = pdb.gimp_gradients_get_list("")
        else:
          res_list = []    
          
        sort_list = []
        sort_list.extend(res_list)
        sort_list.sort()
        root = gtk.Menu()
        menu = gtk.Menu()
        mitem = gtk.MenuItem("#")
        root.append(mitem)
        mitem.show()
        mitem.set_submenu(menu)
          
        num = 0
        for res in sort_list:
          f_res = re.sub("[^\w# ]","",res)  
          child = gtk.MenuItem(f_res)
          if num < self.MENU_LIMIT:
            num = num +1
          else:
            num = 0
            menu = gtk.Menu()
            mitem.set_label(mitem.get_label() +" - "+ res[0])
            mitem = gtk.MenuItem(res[0])
            root.append(mitem)
            mitem.show()
            mitem.set_submenu(menu)      
                
          menu.append(child)
          child.show()   
          child.connect("activate", self.menuitem_activate) 
            
        mitem  = root.get_children()[0] 
        submenu= mitem.get_submenu()
        if submenu is not None:
          first = submenu.get_children()[0]
          last  = submenu.get_children()[submenu.get_children().__len__() -1]
          if first is not None:
            f_letter= first.get_label()[0]
            l_letter= last.get_label()[0]
            mitem.set_label(f_letter +" - "+ l_letter)  
      except Exception, e:
          menu = gtk.Menu()
          mitem = gtk.MenuItem("Resource Chooser say: {0}".format(e))
          menu.append(mitem)        
          mitem.show()
          return menu
      else:    
          return root    
    
    def get_directory(self):
        response = ""
        dialog = gtk.FileChooserDialog(self.window_title, None,
          gtk.FILE_CHOOSER_ACTION_OPEN,
          (
                  gtk.STOCK_CANCEL,
                  gtk.RESPONSE_CANCEL,
                  gtk.STOCK_ADD,
                  gtk.RESPONSE_OK
          )
        )
        dialog.set_default_response(gtk.RESPONSE_OK)
        dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
 
        _filter = gtk.FileFilter()
        _filter.set_name(i18n_const("filter_local_folder"))
        _filter.add_pattern(".")
        dialog.add_filter(_filter) 
    
        if (dialog.run() == gtk.RESPONSE_OK) :
                response = dialog.get_filename()
        dialog.destroy()
    
        return response
   
    def get_resource(self):
        return self.resource.get_text()
     
    def set_resource(self, res_kind, data):
        if res_kind in self.KIND:
            self.resource.set_text(data)
       
    def set_title(self, title):     
        self.window_title = title.capitalize()
    
    def release(self, widget, event):
        if self.resource_kind == "dirname":
            outDirectory = re.sub("\\\\", "/", self.get_directory())
            self.resource.set_text(outDirectory)
      
        if self.resource_kind == "resname":  
            self.menu.popup(None, None, None, 0, 0)            
  
    def changed(self, entry):
        if re.match(self.REGEX[self.resource_kind], entry.get_text()):
            entry.set_icon_from_stock(0, None)
        else:  
            entry.set_icon_from_stock(0, gtk.STOCK_DIALOG_ERROR)

    def menuitem_activate(self, widget):
        self.resource.set_text(widget.get_label())


class ProcedureChooserDialog(gtk.Dialog):
    
    def __init__(self):
        gtk.Dialog.__init__(self  
                            ,i18n_const("pdb_dialog")
                            ,None
                            ,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                             gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        
        self.set_default_response(gtk.RESPONSE_CANCEL)

        # initiate the variable which will be returned after closing
        self._response = ""

        # create a TreeStore with one string column to use as the model
        self.treestore = gtk.TreeStore(str)

        # iterate over procedure database
        gpiter = self.treestore.append(None ,[i18n_const("pdb_buildin")])
        opiter = self.treestore.append(None ,[i18n_const("pdb_imported")])

        for name in SC_PROCS:
            if name.startswith("gimp-"):
                self.treestore.append(gpiter ,[name])
            else:
                self.treestore.append(opiter ,[name])                

        # create the TreeView using treestore
        self.treeview = gtk.TreeView(self.treestore)

        # create the TreeViewColumn to display the data
        self.tvcolumn = gtk.TreeViewColumn()

        # add tvcolumn to treeview
        self.treeview.append_column(self.tvcolumn)

        # create a CellRendererText to render the data
        self.cell = gtk.CellRendererText()

        # add the cell to the tvcolumn and allow it to expand
        self.tvcolumn.pack_start(self.cell, True)

        # set the cell "text" attribute to column 0 - retrieve text from that column in treestore
        self.tvcolumn.add_attribute(self.cell, 'text', 0)

        # make it searchable
        self.treeview.set_search_column(0)

        # Allow sorting on the column
        self.tvcolumn.set_sort_column_id(0)      
        self.tvcolumn.clicked()  
        
        # Add vertical scrollbar
        swH = gtk.ScrolledWindow()
        swH.set_size_request(300 ,300)
        swH.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        swH.add(self.treeview)        
        
        # Configure place for display informations about selected procedure(item)
        self.prochelp = gtk.Label(i18n_const("pdb_dialog_info"))
        self.prochelp.set_line_wrap(True)
        self.prochelp.set_size_request(280,300)
        self.prochelp.set_alignment(xalign=0.2, yalign=0.1)        
        
        # Add to main container
        hbox = gtk.HBox(True)
        hbox.add(swH)
        hbox.add(self.prochelp)
        
        ## Add scrolled window with tree view   
        self.vbox.add(hbox)
        
        ## Actions
        self.treeview.connect('row-activated', self.row_activated)
        self.treeview.connect('move-cursor', self.row_activated)
        
        ## Show all widgets
        self.show_all()

    def row_activated(self, treeview, arg1, arg2):
        (model, seliter) = treeview.get_selection().get_selected()
        
        name_of_data = model.get_value(seliter, 0)
        try:
            res = pdb.query(name_of_data)
            if res:
                proc = pdb[res[0]]
        # choose between full description and tooltip
                proc_description = proc.proc_help if proc.proc_help.__len__() > 1 else proc.proc_blurb.capitalize()
        # counting words to avoid funny text 
        # http://stackoverflow.com/questions/17507876/trying-to-count-words-in-a-string-python
                words = re.split(r'[^0-9A-Za-z]+',proc_description)
                if words.__len__() <= 4:
                    proc_description = proc.proc_blurb.capitalize()
                formated_information = "%s\r\n\n%s" % (proc.proc_author, proc_description)
                
                self.prochelp.set_text(formated_information)
                self._response = name_of_data
        except Exception ,ex:
            pdb.gimp_message("Procedure Chooser say: {1}".format(self.__class__.__name__ ,ex))
            
    def get_selected(self):
        return self._response


class BatchScriptComposer(gtk.Window):

    WIDGET_BORDER_SIZE = 10
    FRAME_BORDER_COLOR = "blue"
    
    WIDTH = 650
    HEIGHT= 450
        
    def __init__(self, safemode):
        gtk.Window.__init__(self)
        
        self.set_title(SC_TITLE)
        self.set_resizable(False)
        self.set_border_width(self.WIDGET_BORDER_SIZE/2)
        self.set_size_request(self.WIDTH ,self.HEIGHT)
        
        self.button_x = 1
        self.name_of_previous = ""
        self.output_directory = ""
        
        self.set_events(gtk.gdk.ALL_EVENTS_MASK)
        self.connect("delete-event" ,self.delete_event)
        self.connect("destroy" ,self.destroy_event)        
            
        # The INI file format parser
        self.config = ConfigParser.ConfigParser()    
            
        # ToolTip Instance
        self.tooltips = gtk.Tooltips()    
            
        # Main layout
        frame_set = gtk.Frame(i18n_const("frame_1"))
        frame_set.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        frame_set.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(self.FRAME_BORDER_COLOR))             
        frame_set.set_size_request(self.WIDTH ,96)   
        
        frame_options = gtk.Frame(i18n_const("frame_2"))
        frame_options.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        frame_options.modify_bg(gtk.STATE_NORMAL ,gtk.gdk.color_parse(self.FRAME_BORDER_COLOR))   
         
        self.messagebox = gtk.Label(SC_SAVEMODE_WARNING) 
         
        hbox = gtk.HBox(True)  
        vbox = gtk.VBox()
        vbox.pack_start(self.messagebox ,False ,False)
        vbox.pack_start(frame_set ,False)
        vbox.add(hbox)   
        
        # Files
        toolbar = gtk.Toolbar()
        toolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
        toolbar.set_style(gtk.TOOLBAR_BOTH_HORIZ) 
        
        stock_add = gtk.image_new_from_stock(gtk.STOCK_ADD ,gtk.ICON_SIZE_SMALL_TOOLBAR)
        stock_clear = gtk.image_new_from_stock(gtk.STOCK_CLEAR ,gtk.ICON_SIZE_SMALL_TOOLBAR)
        stock_convert = gtk.image_new_from_stock(gtk.STOCK_CONVERT ,gtk.ICON_SIZE_SMALL_TOOLBAR)
        
        toolbar.append_item(i18n_const("button_1") ,None ,"Private" ,stock_add ,self.toolbar_action ,SC_ADD_RESOURCES)
        toolbar.append_item(i18n_const("button_2") ,None ,"Private" ,stock_convert ,self.toolbar_action ,SC_ADD_OPENED_IMAGES)
        toolbar.append_item(i18n_const("button_3") ,None ,"Private" ,stock_clear ,self.toolbar_action ,SC_CLEAR_LIST)
        toolbar.show_all()
        
        mn = gtk.TreeViewColumn(None ,gtk.CellRendererText() ,text=0)
        
        self.tree = gtk.TreeView(gtk.ListStore(gobject.TYPE_STRING))
        self.tree.set_headers_visible(False)
        self.tree.append_column(mn)
        
        swH = gtk.ScrolledWindow()
        swH.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        swH.add(self.tree)        
        
        box_files = gtk.VBox()
        box_files.pack_start(toolbar ,False ,False ,5)
        box_files.pack_end(swH ,True ,True)
        
        hbox.add(box_files)
        hbox.add(frame_options)      
        
        # Options
        box_options = gtk.VBox()

        self.button_output = gtk.MenuToolButton(None ,i18n_const("option_output_button"))
        self.button_output.connect("clicked",self.output_action)
        
        self.box_params = gtk.VBox()
        
        swH = gtk.ScrolledWindow()
        swH.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        swH.add_with_viewport(self.box_params)        
        
        swH.get_child().set_shadow_type(gtk.SHADOW_NONE)
        
        box_options.set_border_width(self.WIDGET_BORDER_SIZE)
        box_options.pack_start(self.button_output ,False ,False)
        box_options.pack_end(swH)
        box_options.show_all()
        
        frame_options.add(box_options)
        
        # Sets
        self.button_addset = gtk.Button(None ,gtk.STOCK_ADD)
        self.button_addset.set_size_request(72 ,72) 
        self.button_addset.set_image_position(gtk.POS_TOP)
        self.button_addset.connect("clicked",self.addset_action)
        
        self.container = gtk.Fixed()
        self.container.add(self.button_addset)
        
        frame_set.add(self.container)
        
        # Action Area of Manipulation Set
        button_box = gtk.HButtonBox()
        button_box.set_layout(gtk.BUTTONBOX_END)
        button_box.set_spacing(10)
        
        button = gtk.Button(stock=gtk.STOCK_OPEN)
        _button= gtk.Button(stock=gtk.STOCK_REMOVE)
        self.button_save = gtk.Button(stock=gtk.STOCK_SAVE_AS)
        self.button_execute = gtk.Button(stock=gtk.STOCK_EXECUTE)
        
        self.button_save.set_state(gtk.STATE_INSENSITIVE)
        self.button_execute.set_state(gtk.STATE_INSENSITIVE)
        
        button_box.add(button)
        button_box.add(self.button_save)
        button_box.add(_button)
        button_box.add(self.button_execute)
        
        button.connect("clicked",self.saveload,gtk.FILE_CHOOSER_ACTION_OPEN)
        _button.connect("clicked",self.clearset_action)
        self.button_save.connect("clicked",self.saveload,gtk.FILE_CHOOSER_ACTION_SAVE)
        self.button_execute.connect("clicked",self.execute_action)
        
        vbox.pack_end(button_box ,False)
        
        # Progress
        self.pbar = gtk.ProgressBar()
        vbox.pack_end(self.pbar ,False)
                
        # Show content of main layout
        self.add(vbox) 
        self.show_all()
        if not safemode:
            self.messagebox.hide()
        
    def main(self):
        """ 
        This is a recent change in GTK - the developers wanted icons not to appear on buttons. 
          On Linux, this can be changed by editing the gconf key
          On windows, I think that you need to set a value in your gtkrc file and use a theme that supports icons
          
        The Python equivalent for setting the property without having to change any system configuration files is:
        """
        settings = gtk.settings_get_default()
        settings.props.gtk_button_images = True    
        
        """
        Start GTK main loop
        """
        gtk.main()

    def delete_event(self, widget, event, data=None):
        return False

    def destroy_event(self, widget, data=None):
        gtk.main_quit()

    def addset_action(self, widget):
        button_count = SC_BUTTONS.__len__()
        if button_count < 7: 
            dialog = ProcedureChooserDialog()            
            if (dialog.run() == gtk.RESPONSE_ACCEPT) :
                response = dialog.get_selected()
                if response:
                    try:
                        self.create_buttonandmenu(response)                        
                    except Exception, ex:
                        pdb.gimp_message("{0}[addset_action] say: {1}".format(self.__class__.__name__ ,ex))
                    else:
                        self.button_save.set_sensitive(True)
                        self.button_execute.set_sensitive(True)
            dialog.destroy()
                  
    def clearset_action(self, widget):
        try:
            self.button_addset.hide()
            for child in self.container.get_children():
                if child.get_visible():
                    self.container.remove(child)
                    SC_BUTTONS.remove(child)
                    del child
            
            for child in self.box_params.get_children():
                self.box_params.remove(child)
                del child
            
            self.button_x = 1    
            self.container.move(self.button_addset ,1 ,0)      
            self.button_addset.show()
        except Exception ,ex:
            pdb.gimp_message("{0}[clearset_action] say: {1}".format(self.__class__.__name__ ,ex))  
        else:
            self.button_execute.set_sensitive(False)
            self.button_save.set_sensitive(False)              
                  
    def menuitem_remove(self ,menuitem ,button):
        try:
            index = SC_BUTTONS.index(button)
        except:
            pass
        else:
            for num in range(index,SC_BUTTONS.__len__()):
                x = self.container.child_get_property(SC_BUTTONS[num] ,"x")
                self.container.move(SC_BUTTONS[num] ,x-72 ,0)
    
            self.container.remove(button)
            SC_BUTTONS.remove(button)
            del button
             
            self.container.move(self.button_addset ,x ,0)  
            self.button_x = x 
        
    def button_press(self, widget, event):
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
            widget.popup(None, None, None, event.button, event.time)
            pass    
        if (event.type == gtk.gdk.KEY_PRESS and event.keyval in [32,65293]) or (event.type == gtk.gdk.BUTTON_PRESS and event.button == 1):
            try:
                SC_VALS[self.name_of_previous] = []
                
                for child in self.box_params.children():
                # Data instance
                    entry = child.get_children()[0] if hasattr(child, "get_children") else child
                    class_name = entry.__class__.__name__.lower() 
                # Collect data  
                    if class_name == "entry":
                        SC_VALS[self.name_of_previous].append( entry.get_text() )
                    elif class_name == "combobox":
                        SC_VALS[self.name_of_previous].append( entry.get_active() )  
                    elif class_name == "resourcechooserbutton":
                        SC_VALS[self.name_of_previous].append( entry.get_resource()  )
                    elif class_name == "checkbutton":
                        SC_VALS[self.name_of_previous].append( (0, 1)[entry.get_active()]  )
                    elif class_name == "fontbutton":
                        SC_VALS[self.name_of_previous].append( entry.get_font_name() )
                    elif class_name == "colorbutton":
                        SC_VALS[self.name_of_previous].append( entry.get_color() )                    
                # Clear working area of Property Manager  
                    self.box_params.remove(child)
                    del child
                    
                # Name of class of event trigger 
                cname = widget.__class__.__name__.lower()
                # Choose resource
                if cname == 'menu':
                    menuitem = widget.get_children()[0]
                    name_of_data = menuitem.get_label()
                elif cname == 'button':
                    name_of_data = widget.name_of_data
                else:
                    return      
                self.name_of_previous = name_of_data
                
                # Choose the array of values to put for editing
                values = SC_VALS[name_of_data]
                nvalues= 0
                
                if pdb[name_of_data].nparams > 0:
                # Parameter's Collection
                    params = pdb[name_of_data].params
                # Collect number of editable parameters
                    nvparams = pdb[name_of_data].nparams

                    for p in params:
                    ##PROLOG
                        p_type = p[1].lower()
                        p_length = 0 if p_type in ['toggle'] else 16
                        p_label= text2SuitableTip(p[2],p_length) if p_type in ['value','string','toggle','option'] else value2SuitableLabel(p[1])
                    ##RUN-MODE
                        if p_type == 'run-mode':
                            nvparams -= 1
                            continue
                    ##IMAGE
                        if p_type == 'image':
                            nvparams -= 1
                            continue
                    ##DRAWABLE, LAYER
                        if p_type in ['drawable','layer']:
                            nvparams -= 1
                            continue
                    ##TOGGLE
                        if p_type == 'toggle':
                            entry = gtk.CheckButton(p_label) 
                            if nvalues < values.__len__():
                                entry.set_active(values[nvalues])
                                nvalues += 1
                    ##COLOR
                        elif p_type == 'color':
                            entry = gtk.ColorButton()
                            entry.set_title(p_label)
                            entry.set_size_request(150,25) 
                            if nvalues < values.__len__():
                                val = values[nvalues]
                                try:                                
                                    mycolor = gtk.gdk.color_parse(val) if val.__class__.__name__.lower() == 'str' else val
                                    entry.set_color(mycolor)
                                except:
                                    entry.set_color(gtk.gdk.Color())
                                nvalues += 1
                    ##FONT
                        elif p_type == 'font':
                            entry = gtk.FontButton() 
                            entry.set_title(p_label)
                            entry.set_show_style(False)
                            if nvalues < values.__len__():
                                entry.set_font_name(values[nvalues])
                                nvalues += 1
                    ##GRADIENT
                        elif p_type == 'gradient':
                            entry = ResourceChooserButton(p_type)
                            if nvalues < values.__len__():
                                entry.set_resource("resname", values[nvalues]) 
                                nvalues += 1
                    ##PATTERN
                        elif p_type == 'pattern':
                            entry = ResourceChooserButton(p_type)
                            if nvalues < values.__len__():
                                entry.set_resource("resname", values[nvalues]) 
                                nvalues += 1
                    ##DIRECTORY
                        elif p_type == 'dirname':
                            entry = ResourceChooserButton(p_type)
                            if nvalues < values.__len__():
                                entry.set_resource("dirname", values[nvalues]) 
                                nvalues += 1
                    ##FILENAME
                        elif p_type == 'filename' or p_type.endswith("-filename"):
                            entry = ResourceChooserButton('filename')
                            if nvalues < values.__len__():
                                entry.set_resource("filename", values[nvalues]) 
                                nvalues += 1
                    ##OTHERS
                        else:
                            entry = gtk.Entry()
                            entry.set_size_request(150,20) 
                            if nvalues < values.__len__():  
                                entry.set_text(values[nvalues])
                                nvalues += 1   
                # Set properties
                        entry.hide_on_delete()
                # Add tooltip 
                        if p[2]:
                            self.tooltips.set_tip(entry,text2SuitableTip(p[2]))        
                # Add widgets    
                        hbox = gtk.HBox()
                        hbox.pack_start(entry ,False)
                        if not p_type in ['toggle']:
                            hbox.pack_end(gtk.Label(p_label))
                        self.box_params.pack_start(hbox ,False ,False ,5)
                # Display message
                    if nvparams < 1:
                        self.box_params.add(gtk.Label(i18n_const("pdb_without_input_params")))    
                else:
                # Display message
                    self.box_params.add(gtk.Label(i18n_const("pdb_without_input_params")))
                # Show children
                self.box_params.show_all()
            except Exception, ex:
                pdb.gimp_message("{0}[button_press] say: {1}".format(self.__class__.__name__ ,ex))        
     
    def button_move(self ,menuitem ,button):
        c_x = self.container.child_get_property(button ,"x")
        x = self.container.child_get_property(self.button_addset ,"x")
        
        try:
            index = SC_BUTTONS.index(button)
        except:
            pass
        else:
            n_button = SC_BUTTONS[index+1]            

            
    def output_action(self, widget):
        patch = get_paths(i18n_const("option_output_dialog") ,gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER ,False)
        if patch:
            self.output_directory = patch[0]
            self.button_output.set_label(os.path.split(patch[0])[1])

    def toolbar_action(self, widget, data):
        model = self.tree.get_model()
        if data == SC_ADD_RESOURCES:
            resList = get_paths(i18n_const("button_1") ,gtk.FILE_CHOOSER_ACTION_OPEN ,True)
            if resList:
                for filepath in resList:
                    if os.path.isdir(filepath):
                        for name in os.listdir(filepath):
                            srcpath = os.path.join(filepath ,name)
                            if os.path.isfile(srcpath):
                                SC_FILES[name]=srcpath
                                model.append([name])
                    else:        
                        filename = os.path.basename(filepath)
                        SC_FILES[filename]=filepath
                        model.append([filename])
        elif data == SC_ADD_OPENED_IMAGES:
            for image in gimp.image_list():
                filepath = pdb.gimp_image_get_filename(image)
                filename = os.path.basename(filepath)
                if not SC_FILES.has_key(filename):
                    SC_FILES[filename]=filepath
                    model.append([filename])
        elif data == SC_CLEAR_LIST:
            model.clear()
            SC_FILES.clear()
       
    def execute_action(self, widget):
        nfiles = 0
        self.pbar.set_fraction(0.0)
        if os.path.isdir(self.output_directory):
            try:
                for filename ,filepath in SC_FILES.iteritems():
                    progress = round( abs(float(nfiles) / SC_FILES.__len__()) ,2 )
                    
                    self.pbar.set_fraction( progress )
                    self.pbar.set_text(filename)
                    nfiles -= 1
                    try:
                        image = pdb.gimp_file_load(filepath ,filepath)
                    except Exception ,ex:
                        pdb.gimp_message("Loader say: {1}".format(ex))
                    else:
                        destpath = os.path.join( self.output_directory ,filename )
                        drawable = pdb.gimp_image_get_active_drawable(image)
                        layer = pdb.gimp_image_get_active_layer(image)
                        
                        for button in SC_BUTTONS:
                            name = button.name_of_data
                            item = SC_VALS[name]
                            
                            attr = []
                            num  = 0
                            for param in pdb[name].params:
                                if param[1] == 'image':
                                    attr.append(image)
                                elif param[1] == 'drawable':
                                    attr.append(drawable)
                                elif param[1] == 'layer':
                                    attr.append(layer)
                                else:
                                    if num < item.__len__():
                                        attr.append(item[num])
                                        num += 1
                                        
                            try:
                                pdb[name](*attr)
                            except Exception ,ex:
                                pdb.gimp_message("Executor say: {1}".format(ex))
                        try:        
                            pdb.gimp_file_save(image ,drawable ,destpath ,destpath)
                        except Exception ,ex:
                            pdb.gimp_message("Saver say: {1}".format(ex))
            except Exception ,ex:
                pdb.gimp_message("Executor say: {1}".format(ex))
            else:
                self.pbar.set_fraction(1.00)
           
    def create_buttonandmenu(self ,proc_name):
        button = gtk.Button()
        button.name_of_data = proc_name
        
        button.set_image(gtk.image_new_from_stock(gtk.STOCK_EXECUTE ,gtk.ICON_SIZE_BUTTON))
        button.set_size_request(72 ,72)  
        button.hide_on_delete()
        button.show()
        
        menu = gtk.Menu()
    
        item_3 = gtk.MenuItem(i18n_const("menuitem_moveforward"))
        item_2 = gtk.MenuItem(i18n_const("menuitem_removeme"))
        item_1 = gtk.MenuItem(proc_name)
        
        menu.append(item_1)
        menu.append(item_2)
        menu.append(item_3)
        
        item_1.set_state(gtk.STATE_INSENSITIVE)
        item_2.connect("activate", self.menuitem_remove ,button)
        item_3.connect("activate",self.button_move ,button)
        
        menu.show_all()
         
        SC_BUTTONS.append(button)
        SC_VALS[proc_name] = []
        
        button.connect_object("event" ,self.button_press ,menu)

        self.button_x += 72
        self.container.move(self.button_addset ,self.button_x ,0)
        self.container.put(button ,self.button_x-72 ,0)        
        button.grab_focus()
        
        self.tooltips.set_tip(button,pdb[proc_name].proc_blurb)
           
    def saveload(self, widget, action):
        paths = get_paths(i18n_const("saveload_dialog") ,action ,False)
        if paths:
            filepath = os.path.splitext(paths[0])[0] + ".bimp".lower()
            try:
                if action == gtk.FILE_CHOOSER_ACTION_SAVE:
                    block_counter = 0
                    filename= slugify(os.path.basename(filepath))
                    filepath= os.path.join( os.path.dirname(filepath) , filename )
                    with open(filepath,"w") as f:
                        f.write("#BIMP compatible\n")
                        f.write("#MANIPULATION SET DEFINITION")
                        for btn in SC_BUTTONS:
                            block_name = btn.name_of_data
                            f.write("\n\n[USERDEF%d]\n" % block_counter)
                            f.write("procedure=%s\n" % block_name)
                            f.write("num_params=%d\n" % pdb[block_name].nparams)
                            block_counter += 1
                            param_counter = 0  
                            val_counter = 0
                            params = pdb[block_name].params
                            for p in params:
                                if p[1] in SC_EXCLUDE_TYPE:
                                    val = "NOT_USED"
                                else:
                                    val = SC_VALS[block_name][val_counter]
                                    val_counter += 1
                                    
                                if p[1] == 'color':
                                    val = val.to_string().upper()
                                f.write( "PARAM{0}={1}\n".format (param_counter ,val) )
                                param_counter += 1
                    f.close()
                    
                if action == gtk.FILE_CHOOSER_ACTION_OPEN:
                    if os.path.isfile(filepath) and SC_BUTTONS.__len__() < 6:
                        self.config.read(filepath)
                        for section in self.config.sections():
                        # Create button
                            proc_name = self.config.get(section ,"procedure")
                            self.create_buttonandmenu(proc_name)
                        # Read property values
                            SC_VALS[proc_name] = []
                            nparams = self.config.get(section , "num_params")
                            if nparams:
                                for num in range(int(nparams)):
                                    try:
                                        val = self.config.get(section , "PARAM%d" % int(num))
                                    except (ValueError ,TypeError) ,ex:
                                        pdb.gimp_message("Loader say: {0}".format(ex))
                                        val = ""    
                                    else:
                                        val = int(val) if val.isdigit() else val    
                                        
                                    if val != "NOT_USED":
                                        SC_VALS[proc_name].append(val)

                    
            except Exception ,ex:
                pdb.gimp_message("{0}[saveload] say: {1}".format(self.__class__.__name__ ,ex))

            if SC_BUTTONS.__len__() > 0:
                event = gtk.gdk.Event(gtk.gdk.KEY_PRESS)
                event.keyval = 32
                self.button_press(SC_BUTTONS[0], event)
                SC_BUTTONS[0].grab_focus()
                        
            self.button_execute.set_sensitive(True)
            self.button_save.set_sensitive(True)
   
 
def i18n_const(key):
    if SC_I18N.has_key(key):
        return SC_I18N[key] 
    else:
        return key   

def value2SuitableLabel(data):
    return re.sub('[-_]', ' ', data).strip().lower().capitalize()

def text2SuitableTip(data ,length=0):
    """
    Replace with pattern, remove white chars and capitalize
    """
    patterns = {"_":""}
    formated = data
    for arg1 , arg2 in patterns.iteritems():
        formated = re.sub(arg1 ,arg2 ,formated ,0 ,re.IGNORECASE)
    
    formated = formated.strip().capitalize()
    ff = formated.split()
    if ff[0].__len__() < 4:
        ff[0] = ff[0].upper()
        formated = " ".join(ff)

    # If it is given the maximum length of the string that divide so that all the words were, 
    # on the occasion of removing everything that is not a letter
    if length:
        nlabel = 0
        words = formated.split(" ")
        accepted = []
        for word in words:
            nlabel += word.__len__()
            new_word = re.sub('[^-a-zA-Z]','',word).strip()
            if new_word: 
                accepted.append( new_word ) 
            if nlabel > length:
                break
    
        formated = " ".join(accepted)
                                    
    return formated

def slugify(value):
    """
    Normalizes string, converts to lowercase, removes non-alpha characters,
    and converts spaces to hyphens.

    Based on code in: Django
    """
    value = unicodedata.normalize('NFKD', unicode(value))
    value = value.encode('ascii', 'ignore')
    value = re.sub('[^\w\s.-]', '', value).strip().lower()
    value = re.sub('[-\s]+', '-', value)
    return value

def multiple_replace(dict, text):
  """
  This recipe shows how to use the Python standard re module to perform 
  single-pass multiple string substitution using a dictionary.
  
  http://code.activestate.com/recipes/81330-single-pass-multiple-replace
  """  
  # Create a regular expression  from the dictionary keys
  regex = re.compile("(%s)" % "|".join(map(re.escape, dict.keys())))

  # For each match, look-up corresponding value in dictionary
  return regex.sub(lambda mo: dict[mo.string[mo.start():mo.end()]], text) 
  

def get_paths(title, action ,multiple):
    response = ""
    try:
        dialog = gtk.FileChooserDialog(
                title
                ,None
                ,action
                ,(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                  gtk.STOCK_APPLY, gtk.RESPONSE_ACCEPT)
        )
        dialog.set_select_multiple(multiple)
        dialog.set_default_response(gtk.RESPONSE_ACCEPT)
  
        if action == gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER:
            _filter = gtk.FileFilter()
            _filter.set_name(i18n_const("filter_local_folder"))
            _filter.add_pattern(".")
            dialog.add_filter(_filter)
 
        if (dialog.run() == gtk.RESPONSE_ACCEPT) :
                response = dialog.get_filenames()
        dialog.destroy()
    except Exception ,ex:
        pdb.gimp_message("{0}[get_paths] say: {1}".format(ex.__class__.__name__ ,ex))
    else:
        return response


def runner():
    
    # Gimp preferences
    SC_GIMPVERSION = os.path.splitext(pdb.gimp_version())[0]
    SC_CONFIGFILE = os.path.join(os.environ["userprofile"] ,".gimp-"+SC_GIMPVERSION ,"gimprc")
    
    if os.path.isfile(SC_CONFIGFILE) and SC_PREF.__len__() == 0:
        namepattern = "[a-zA-Z0-9_-]+[ ]+"
        regexp = re.compile(r'^\(%s.+\)$' % namepattern ,re.IGNORECASE)
        with open(SC_CONFIGFILE ,"r") as f:
            for line in f.readlines():
                line = line.strip()
                if regexp.match(line):
                    temp = re.search(namepattern, line[1:-1])
                    value = line[temp.end():-1].strip().replace('"','').split(os.pathsep)
                    
                    if value.__len__() == 1:
                        value = value[0]
                    SC_PREF[temp.group().strip()] = value
    
    # Collect Gimp procedures
    regex = re.compile(r'Use(.){1,}[\'"-a-z]+[\s]+instead.$', re.IGNORECASE)
    for name in gimp.pdb.query():
        if regex.search( gimp.pdb[name].proc_blurb ):
            continue
        SC_PROCS.append(name)
    
    # Internationalization
    safemode = False
    if SC_PREF.has_key("language"):
        SC_PREF["language"] = SC_PREF["language"][:2]
    else:
        SC_PREF["language"] = "en"
        safemode = True
    SC_LANGFILE = os.path.join(SC_RESDIR ,"locale" ,SC_PREF["language"].lower() +".json")
    SC_I18N.clear()
    
    if os.path.isfile(SC_LANGFILE):
        try:
            text = open(SC_LANGFILE ,"r").read()
            text = text.decode("windows-1250")
            matrix = json.loads(text)
        except IOError:
            pass
        else:
            SC_I18N.update(matrix)
    
    # Initialize plug-in                          
    BatchScriptComposer(safemode).main()
    return None
 

register(
         "python-fu-batch-scriptcomposer" 
        , "Complete programming environment to automated Gimp operations"
        , "Complete programming environment to automated Gimp operations." +
          " Applies GIMP manipulations on groups of images. Deprecated procedures in your version of GIMP is excluded from use."
        , "Jeremy Czajkowski"
        , "Jeremy Czajkowski"
        , "September 2013"
        , "<Toolbox>/_Tools/" + SC_TITLE 
        , None
        , []
        , []
        , runner)

if __name__ == "__main__":
    main()
